dp27projectileHandler = {
	shellNum = 1,
	shells = {},
	defaultShell = {active = false},
}

function init()
	toolmodel = {}
	drummag = GetBool("savegame.mod.drumMag")
	if drummag == true then
		toolmodel = "MOD/vox/DT27.vox"
		toolname = "DT-27"
	else
		toolmodel = "MOD/vox/DP27.vox"
		toolname = "DP-27"
	end
	
	RegisterTool("dp27", toolname, toolmodel)
	SetBool("game.tool.dp27.enabled", true)
	SetFloat("game.tool.dp27.ammo", 101)
	
	unlimitedammo = GetBool("savegame.mod.unlimitedammo")
	if drummag == true then
		magsize = 63
	else
		magsize = 47
	end
	
	if drummag == true then
		magcount = 5
	else
		magcount = 7
	end
	
	damage = 1.09
	gravity = Vec(0, 0, 0)
	velocity = 1.87
	reloadTime = 2.5
	shotDelay = 60/550
	spreadTimer = 0
	ammo = magsize
	mags = magcount
	reloading = false
	
	
	for i=1, ammo do
		dp27projectileHandler.shells[i] = deepcopy(dp27projectileHandler.defaultShell)
	end

	shootTimer = 0
	reloadTimer = 0
	recoilTimer = 0
	lightTimer = 0
	if drummag == true then
		gunsound = LoadSound("MOD/snd/dt27fire.ogg")
	else
		gunsound = LoadSound("MOD/snd/dp27fire.ogg")
	end
	cocksound = LoadSound("MOD/snd/bolt.ogg")
	reloadsound = LoadSound("MOD/snd/mag_out.ogg")
	dryfiresound = LoadSound("MOD/snd/dryfire.ogg")
	refillsound = LoadSound("MOD/snd/refill.ogg")
end

function canShoot()
	local vehicle = GetPlayerVehicle()
	if vehicle ~= 0 then
		local driverPos = GetVehicleDriverPos(vehicle)
		local t = GetVehicleTransform(vehicle)
		local worldPos = TransformToParentPoint(t, driverPos)
		local cameraPos = GetCameraTransform().pos
		local length = VecLength(VecSub(cameraPos, worldPos))

		if length < 1 then
			return true
		else
			return false
		end
	end
	return true
end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else
        copy = orig
    end
    return copy
end

function GetAimPos()
	local ct = GetCameraTransform()
	local forwardPos = TransformToParentPoint(ct, Vec(0, 0, -100))
    local direction = VecSub(forwardPos, ct.pos)
    local distance = VecLength(direction)
	local direction = VecNormalize(direction)
	local hit, hitDistance = QueryRaycast(ct.pos, direction, distance)
	if hit then
		forwardPos = TransformToParentPoint(ct, Vec(0, 0, -hitDistance))
		distance = hitDistance
	end
	return forwardPos, hit, distance
end

function Shoot()
	if shootTimer > 0 or ammo == 0 then
		return
	end
	aimpos, hit, distance = GetAimPos()

	local p = toolPos
	local dir = VecSub(aimpos, p)
	local maxSpread = InputDown("ctrl") and 2 or 4
	local spread = math.min(spreadTimer, maxSpread) * distance/100
	dir[1] = dir[1] + (math.random()-0.5)*2*spread
	dir[2] = dir[2] + (math.random()-0.5)*2*spread
	dir[3] = dir[3] + (math.random()-0.5)*2*spread
	
	dp27projectileHandler.shells[dp27projectileHandler.shellNum] = deepcopy(dp27projectileHandler.defaultShell)
	loadedShell = dp27projectileHandler.shells[dp27projectileHandler.shellNum] 
	loadedShell.active = true
	loadedShell.pos = p
	loadedShell.predictedBulletVelocity = VecScale(dir, velocity*(100/distance))
	dp27projectileHandler.shellNum = (dp27projectileHandler.shellNum%#dp27projectileHandler.shells) + 1
	
	SpawnParticle("fire", toolPos, Vec(0, 1.3+math.random(1,10)*0.1, 0), 0.3, 1)
	PlaySound(gunsound, GetPlayerTransform().pos, 0.9, false)

	if not unlimitedammo then
		ammo = ammo - 1
	end
	shootTimer = shotDelay
	recoilTimer = shotDelay
	lightTimer = shotDelay/2
	spreadTimer = spreadTimer + 1.25
end

function ProjectileOperations(projectile)
	projectile.predictedBulletVelocity = VecAdd(projectile.predictedBulletVelocity, (VecScale(gravity, GetTimeStep())))
	local point2 = VecAdd(projectile.pos, VecScale(projectile.predictedBulletVelocity, GetTimeStep()))
	local dir = VecNormalize(VecSub(point2, projectile.pos))
	local hit, dist = QueryRaycast(projectile.pos, dir, VecLength(VecSub(point2, projectile.pos)))
	
	local hitTransparant, distT = QueryRaycast(projectile.pos, dir, dist * 2, 0, false)
	local hitNonTransparant, distNonT = QueryRaycast(projectile.pos, dir, dist * 2, 0, true)
	local hitWindow = hitTransparant and not hitNonTransparant
	
	if hit then	
		hitPos = VecAdd(projectile.pos, VecScale(VecNormalize(VecSub(point2, projectile.pos)), dist))
		projectile.active = false
		if hitWindow then
			MakeHole(hitPos, 0.5)
		end
		if not hitWindow then
			MakeHole(hitPos, 0.15, 0.075)
		end
	SpawnParticle("smoke", hitPos, Vec(0, 1.0+math.random(1,5)*-0.1, 0), damage, 1)
	else
		DrawLine(projectile.pos, point2)
	end
	projectile.pos = point2
end

function Reload()
	if reloading then
		return
	end
	reloading = true
	PlaySound(reloadsound, GetPlayerTransform().pos, 0.4, false)
	reloadTimer = reloadTime
	mags = mags - 1
end

function tick(dt)
	if GetString("game.player.tool") == "dp27" then
		local b = GetToolBody()
		if b ~= 0 then
			local offset = Transform(Vec(0.2, -1.3, 0))
			local x, y, z, rot = 0.2, -1.3, 0 , 0
			SetToolTransform(offset)
			toolTrans = GetBodyTransform(b)
			toolPos = TransformToParentPoint(toolTrans, Vec(0.45, 0.95, -3.5))
			
			if recoilTimer > 0 then
				local t = Transform()
				t.pos = Vec(x, y, recoilTimer+z)
				ironrot = -recoilTimer*25-rot
				t.rot = QuatEuler(-ironrot, 0, 0)
				SetToolTransform(t)
				
				recoilTimer = recoilTimer - dt
			end
			if lightTimer > 0 then
				if drummag == false then
					PointLight(toolPos, 1, 1, 1, 0.05)
					lightTimer = lightTimer - dt
				else
				PointLight(toolPos, 1, 1, 1, 0.7)
				lightTimer = lightTimer - dt
				end
			end

			if not unlimitedammo then
				if ammo < magsize and mags > 1 and InputPressed("R") then
					Reload()
				end
				
				if GetBool("ammobox.refill") then
					SetBool("ammobox.refill", false)
					mags = mags + 1
					PlaySound(refillsound, GetPlayerTransform().pos, 0.7, false)
				end

				if reloading then
					reloadTimer = reloadTimer - dt
					if reloadTimer < 0 then
						ammo = magsize
						reloadTimer = 0
						PlaySound(cocksound)
						reloading = false
					end
				end
			end
			
			if InputDown("lmb") and ammo > 0 and not reloading then
					Shoot()
			end

			if InputPressed("lmb") and not reloading then
				spreadTimer = 0
				if ammo == 0 then
					PlaySound(dryfiresound, GetPlayerTransform().pos, 0.8, false)
				end
			end

			if InputReleased("lmb") then
				SpawnParticle("darksmoke", toolPos, Vec(0, 1.3+math.random(1,10)*0.1, 0), 0, 1)
			end
			if shootTimer > 0 or ammo == 0 then
					shootTimer = shootTimer - dt
			end
			for key, shell in ipairs(dp27projectileHandler.shells) do
				if shell.active then
					ProjectileOperations(shell)
				end
			end 
		end
	end
end

function draw()
	if GetString("game.player.tool") == "dp27" and GetPlayerVehicle() == 0 and not unlimitedammo then
		UiPush()
			UiTranslate(UiCenter(), UiHeight()-60)
			UiAlign("center middle")
			UiColor(1, 1, 1)
			UiFont("bold.ttf", 32)
			UiTextOutline(0,0,0,1,0.1)
			if reloading then
				UiText("Reloading")
			else
				UiText(ammo.."/"..magsize*math.max(0, mags-1))
			end
		end
		UiPop()
end